<?php
/**
 * @copyright   Copyright (c) http://www.manadev.com
 * @license     http://www.manadev.com/license  Proprietary License
 */

namespace Manadev\ProductCollection\Resources\Facets\Swatch;

use Magento\Framework\App\ObjectManager;
use Magento\Framework\DB\Select;
use Magento\InventorySalesApi\Api\Data\SalesChannelInterface;
use Magento\InventorySalesApi\Api\Data\SalesChannelInterfaceFactory;
use Magento\InventorySalesApi\Api\GetStockBySalesChannelInterface;
use Manadev\ProductCollection\Contracts\Facet;
use Manadev\ProductCollection\Contracts\FacetResource;
use Manadev\ProductCollection\Facets\Swatch\OptimizedFacet;
use Zend_Db;
use Zend_Db_Expr;

class OptimizedFacetResource extends FacetResource
{

    /**
     * @param Select $select
     * @param Facet $facet
     * @return mixed
     */
    public function count(Select $select, Facet $facet)
    {
        /* @var $facet OptimizedFacet */
        $this->prepareSelect($select, $facet);
        $result = $this->getConnection()->fetchAll($select);
        return count($result) ? $result : false;
    }

    /**
     * @param Select $select
     * @param OptimizedFacet $facet
     */
    public function prepareSelect(Select $select, OptimizedFacet $facet)
    {
        $this->helperResource->clearFacetSelect($select);

        $selectedOptionIds = $facet->getSelectedOptionIds();

        if ($selectedOptionIds !== false) {
            return $this->_prepareSelectWithOptionIds($select, $facet, $selectedOptionIds);
        }

        return $this->_prepareSelect($select, $facet);
    }

    /**
     * @param Select $select
     * @param OptimizedFacet $facet
     * @param $selectedOptionIds
     */
    private function _prepareSelectWithOptionIds(Select $select, OptimizedFacet $facet, $selectedOptionIds)
    {
        $db = $this->getConnection();
        $storeId = $this->getStoreId();

        $selectedOptionIds = $facet->getSelectedOptionIds();

        $fields = [
            'value'       => new Zend_Db_Expr("DISTINCT(`eav`.`value`)"),
            'sort_order'  => new Zend_Db_Expr("`o`.`sort_order`"),
            'label'       => new Zend_Db_Expr("COALESCE(`vs`.`value`, `vg`.`value`)"),
            'is_selected' => new Zend_Db_Expr("`eav`.`value` IN (" . implode(',', $selectedOptionIds) . ")"),
            'swatch_type' => new Zend_Db_Expr("COALESCE(`ss`.`type`, `sg`.`type`)"),
            'swatch'      => new Zend_Db_Expr("COALESCE(`ss`.`value`, `sg`.`value`)"),
            'count'       => new Zend_Db_Expr("1")
        ];
        $select
            ->joinInner(
                ['eav' => $this->getTable('catalog_product_index_eav')],
                "`eav`.`entity_id` = `e`.`entity_id` AND
                {$db->quoteInto("`eav`.`attribute_id` = ?", $facet->getAttributeId())} AND
                {$db->quoteInto("`eav`.`store_id` = ?", $storeId)}",
                null
            )
            ->joinInner(
                ['o' => $this->getTable('eav_attribute_option')],
                "`o`.`option_id` = `eav`.`value`",
                null
            )
            ->joinInner(
                ['vg' => $this->getTable('eav_attribute_option_value')],
                $db->quoteInto("`vg`.`option_id` = `eav`.`value` AND `vg`.`store_id` = ?", 0),
                null
            )
            ->joinLeft(
                ['vs' => $this->getTable('eav_attribute_option_value')],
                $db->quoteInto("`vs`.`option_id` = `eav`.`value` AND `vs`.`store_id` = ?", $storeId),
                null
            )
            ->joinInner(
                ['sg' => $this->getTable('eav_attribute_option_swatch')],
                $db->quoteInto("`sg`.`option_id` = `eav`.`value` AND `sg`.`store_id` = ?", 0),
                null
            )
            ->joinLeft(
                ['ss' => $this->getTable('eav_attribute_option_swatch')],
                $db->quoteInto("`ss`.`option_id` = `eav`.`value` AND `ss`.`store_id` = ?", $storeId),
                null
            );

        $inStock = $this->getInStock($facet);

        if (!empty($inStock)) {
            $ids = join(',', $inStock);
            $select->where("`eav`.`value` IN ({$ids})");
        }
        $select->columns($fields)->order('sort_order');

        return $select;
    }

    /**
     *  FOS-545
     *
     * --  ________                          _____ _____
     * --  ___  __ \_____ __________________ __  /___  /_______
     * --  __  /_/ /_  _ \___  /___  /_  __ \_  __/_  __/_  __ \
     * --  _  ____/ /  __/__  /___  /_/ /_/ // /_  / /_  / /_/ /
     * --  /_/      \___/ _____/_____/\____/ \__/  \__/  \____/
     * --
     * @param OptimizedFacet $facet
     * @return array
     */
    public function getInStock(OptimizedFacet $facet)
    {
        $db = $this->getConnection();
        $storeId = $this->getStoreId();
        $attributeId = $facet->getAttributeId();
        $select = $db->select();

        $objectManager = ObjectManager::getInstance();
        if ($this->helperResource->isMultistockAvailable()) {
            $storeManager = $this->helperResource->getStore();
            $salesChannel = $objectManager->get(SalesChannelInterfaceFactory::class);
            $stockChannel = $objectManager->get(GetStockBySalesChannelInterface::class);

            $channel = $salesChannel->create();
            /** @var SalesChannelInterface $channel */
            $channel->setType(SalesChannelInterface::TYPE_WEBSITE);
            $channel->setCode($storeManager->getWebsite()->getCode());
            $stockInfo = $stockChannel->execute($channel);

            $select
                ->from(
                    ['eav' => $this->getTable('catalog_product_index_eav')],
                    ['total' => new Zend_Db_Expr('SUM(so.is_salable)'), 'value' => 'value']
                )
                ->joinLeft(
                    ['cpe' => $this->getTable('catalog_product_entity')],
                    "`cpe`.`entity_id` = `eav`.`source_id`",
                    []
                )
                ->join(
                    ['so' => $this->getTable("inventory_stock_{$stockInfo->getStockId()}")],
                    '`so`.`sku` = `cpe`.`sku`',
                    []
                )
                ->where("attribute_id = {$attributeId} AND store_id = {$storeId}")
                ->group('value');
        } else {
            $select
                ->from(
                    ['eav' => $this->getTable('catalog_product_index_eav')],
                    ['total' => new Zend_Db_Expr('SUM(so.is_in_stock)'), 'value' => 'value']
                )
                ->joinLeft(
                    ['cpe' => $this->getTable('catalog_product_entity')],
                    "`cpe`.`entity_id` = `eav`.`source_id`",
                    []
                )
                ->join(
                    ['so' => $this->getTable('cataloginventory_stock_item')],
                    '`so`.`product_id` = `cpe`.`entity_id`',
                    []
                )
                ->where("attribute_id = {$attributeId} AND store_id = {$storeId}")
                ->group('value');
        }

        $result = $db->fetchAll($select, [], Zend_Db::FETCH_ASSOC);

        $inStock = [];
        foreach ($result as $item) {
            if ((int)$item['total'] > 0) {
                $inStock[] = (int)$item['value'];
            }
        }

        return $inStock;
    }

    /**
     * @param Select $select
     * @param OptimizedFacet $facet
     * @return Select
     */
    private function _prepareSelect(Select $select, OptimizedFacet $facet)
    {
        $db = $this->getConnection();
        $storeId = $this->getStoreId();

        $fields = [
            'value'       => new Zend_Db_Expr("DISTINCT(`eav`.`value`)"),
            'sort_order'  => new Zend_Db_Expr("`o`.`sort_order`"),
            'label'       => new Zend_Db_Expr("COALESCE(`vs`.`value`, `vg`.`value`)"),
            'is_selected' => new Zend_Db_Expr("0"),
            'swatch_type' => new Zend_Db_Expr("COALESCE(`ss`.`type`, `sg`.`type`)"),
            'swatch'      => new Zend_Db_Expr("COALESCE(`ss`.`value`, `sg`.`value`)"),
            'count'       => new Zend_Db_Expr("1")
        ];
        $select
            ->joinInner(
                ['eav' => $this->getTable('catalog_product_index_eav')],
                "`eav`.`entity_id` = `e`.`entity_id` AND
                {$db->quoteInto("`eav`.`attribute_id` = ?", $facet->getAttributeId())} AND
                {$db->quoteInto("`eav`.`store_id` = ?", $storeId)}",
                null
            )
            ->joinInner(
                ['o' => $this->getTable('eav_attribute_option')],
                "`o`.`option_id` = `eav`.`value`",
                null
            )
            ->joinInner(
                ['vg' => $this->getTable('eav_attribute_option_value')],
                $db->quoteInto("`vg`.`option_id` = `eav`.`value` AND `vg`.`store_id` = ?", 0),
                null
            )
            ->joinLeft(
                ['vs' => $this->getTable('eav_attribute_option_value')],
                $db->quoteInto("`vs`.`option_id` = `eav`.`value` AND `vs`.`store_id` = ?", $storeId),
                null
            )
            ->joinInner(
                ['sg' => $this->getTable('eav_attribute_option_swatch')],
                $db->quoteInto("`sg`.`option_id` = `eav`.`value` AND `sg`.`store_id` = ?", 0),
                null
            )
            ->joinLeft(
                ['ss' => $this->getTable('eav_attribute_option_swatch')],
                $db->quoteInto("`ss`.`option_id` = `eav`.`value` AND `ss`.`store_id` = ?", $storeId),
                null
            );

        $inStock = $this->getInStock($facet);

        if (!empty($inStock)) {
            $ids = join(',', $inStock);
            $select->where("`eav`.`value` IN ({$ids})");
        }
        $select->columns($fields)
            ->order('sort_order');
        return $select;
    }

    public function getFilterCallback(Facet $facet)
    {
        return $this->helperResource->dontApplyFilterNamed($facet->getName());
    }

    /**
     * Resource initialization
     *
     * @return void
     */
    protected function _construct()
    {
        $this->_setMainTable('catalog_product_index_eav');
    }
}
