<?php
/**
 * @copyright   Copyright (c) http://www.manadev.com
 * @license     http://www.manadev.com/license  Proprietary License
 */

namespace Manadev\ProductCollection\Resources;

use Magento\Framework\DB\Select;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Model\ResourceModel\Db;
use Magento\Framework\Module\Manager;
use Magento\Framework\Pricing\PriceCurrencyInterface;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Model\StoreManagerInterface;
use Manadev\ProductCollection\Contracts\Filter;

class HelperResource extends Db\AbstractDb
{
    /**
     * @var PriceCurrencyInterface
     */
    protected $priceCurrency;
    /**
     * @var TaxResource
     */
    protected $taxResource;
    protected $priceExpr;
    protected $currencyRate;
    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;
    /**
     * @var Manager
     */
    private $moduleManager;

    public function __construct(
        Db\Context $context,
                                PriceCurrencyInterface $priceCurrency,
                                TaxResource $taxResource,
                                StoreManagerInterface $storeManager,
                                Manager $moduleManager,
                                $resourcePrefix = null
    ) {
        parent::__construct($context, $resourcePrefix);
        $this->priceCurrency = $priceCurrency;
        $this->taxResource = $taxResource;
        $this->storeManager = $storeManager;
        $this->moduleManager = $moduleManager;
    }

    public function getPriceExpression()
    {
        if (!$this->priceExpr) {
            $this->priceExpr = "`price_index`.`min_price`";
            $this->priceExpr = $this->taxResource->applyTaxToPriceExpression($this->priceExpr);
            $this->priceExpr = $this->applyCurrencyRateToPriceExpression($this->priceExpr);
            $this->priceExpr = "ROUND($this->priceExpr, 2)";
        }

        return $this->priceExpr;
    }

    protected function applyCurrencyRateToPriceExpression($priceExpr)
    {
        if ($this->getCurrencyRate() == 1) {
            return $priceExpr;
        }

        return "($priceExpr)*" . round($this->getCurrencyRate(), 4);
    }

    public function getCurrencyRate()
    {
        if ($this->currencyRate === null) {
            $this->currencyRate = $this->storeManager->getStore()->getCurrentCurrencyRate();
            if (!$this->currencyRate) {
                $this->currencyRate = 1;
            }
        }

        return $this->currencyRate;
    }

    public function formatPriceRangeFacet(&$item, $from, $to, $isFirst, $isLast)
    {
        if ($isFirst) {
            $item['label'] = __("Below %1", $this->priceCurrency->format($to));
            if (!isset($item['value'])) {
                $item['value'] = "-{$to}";
            }
        } elseif ($isLast) {
            $item['label'] = __("%1 and above", $this->priceCurrency->format($from));
            if (!isset($item['value'])) {
                $item['value'] = "{$from}-";
            }
        } else {
            $item['label'] = __("%1 - %2", $this->priceCurrency->format($from), $this->priceCurrency->format($to));
            if (!isset($item['value'])) {
                $item['value'] = "{$from}-{$to}";
            }
        }
    }

    public function formatDecimalRangeFacet(&$item, $from, $to, $isFirst, $isLast)
    {
        if ($isFirst) {
            $item['label'] = __("Below %1", $this->formatDecimal($to));
            if (!isset($item['value'])) {
                $item['value'] = "-{$to}";
            }
        } elseif ($isLast) {
            $item['label'] = __("%1 and above", $this->formatDecimal($from));
            if (!isset($item['value'])) {
                $item['value'] = "{$from}-";
            }
        } else {
            $item['label'] = __("%1 - %2", $this->formatDecimal($from), $this->formatDecimal($to));
            if (!isset($item['value'])) {
                $item['value'] = "{$from}-{$to}";
            }
        }
    }

    public function formatDecimal($value)
    {
        return sprintf("%d", round($value));
    }

    public function clearFacetSelect(Select $select)
    {
        $select->reset(Select::COLUMNS);
        $select->reset(Select::ORDER);
        $select->reset(Select::GROUP);
        $select->reset(Select::LIMIT_COUNT);
        $select->reset(Select::LIMIT_OFFSET);
    }

    public function addAppliedRanges(&$counts, $range, $appliedRanges)
    {
        foreach ($appliedRanges as $appliedRange) {
            list($from, $to) = $appliedRange;
            $index = $from === '' ? floor($to / $range) - 1 : floor($from / $range);

            $found = false;

            foreach ($counts as &$item) {
                if ($item['range'] == $index) {
                    $found = true;
                    $item['is_selected'] = true;
                    $item['value'] = "{$from}-{$to}";
                    break;
                }
            }

            if (!$found) {
                $counts[] = ['range' => $index, 'count' => 0, 'is_selected' => true];
            }
        }
        usort($counts, function ($a, $b) {
            if ((int)$a['range'] < (int)$b['range']) {
                return -1;
            }
            if ((int)$a['range'] > (int)$b['range']) {
                return 1;
            }
            return 0;
        });
    }

    public function dontApplyFilterNamed($name)
    {
        return function (Filter $filter) use ($name) {
            return $filter->getFullName() != 'layered_nav_' . $name;
        };
    }

    public function dontApplyLayeredNavigationFilters()
    {
        return function (Filter $filter) {
            return strpos($filter->getFullName(), 'layered_nav_') !== 0;
        };
    }

    /**
     * @return bool
     */
    public function isMultistockAvailable()
    {
        return $this->moduleManager->isEnabled('Magento_Inventory');
    }

    /**
     * @return StoreInterface
     */
    public function getStore()
    {
        try {
            return $this->storeManager->getStore();
        } catch (NoSuchEntityException $e) {
        }
    }

    /**
     * Resource initialization
     *
     * @return void
     */
    protected function _construct()
    {
        $this->_setMainTable('catalog_product_entity');
    }
}
